<?php

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * HELPER COMMENT START
 * 
 * This is the main class that is responsible for registering
 * the core functions, including the files and setting up all features. 
 * 
 * To add a new class, here's what you need to do: 
 * 1. Add your new class within the following folder: core/includes/classes
 * 2. Create a new variable you want to assign the class to (as e.g. public $helpers)
 * 3. Assign the class within the instance() function ( as e.g. self::$instance->helpers = new Uniwin_Fortnox_Helpers();)
 * 4. Register the class you added to core/includes/classes within the includes() function
 * 
 * HELPER COMMENT END
 */

if ( ! class_exists( 'Uniwin_Fortnox' ) ) :

	/**
	 * Main Uniwin_Fortnox Class.
	 *
	 * @package		UNIWINFORT
	 * @subpackage	Classes/Uniwin_Fortnox
	 * @since		1.0.0
	 * @author		Mohan
	 */
	final class Uniwin_Fortnox {

		/**
		 * The real instance
		 *
		 * @access	private
		 * @since	1.0.0
		 * @var		object|Uniwin_Fortnox
		 */
		private static $instance;

		/**
		 * UNIWINFORT helpers object.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @var		object|Uniwin_Fortnox_Helpers
		 */
		public $helpers;

		/**
		 * UNIWINFORT settings object.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @var		object|Uniwin_Fortnox_Settings
		 */
		public $settings;

		/**
		 * UNIWINFORT ufadminnotice object.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @var		object|Adminnotice
		 */
		public $ufadminnotice;

		

		/**
		 * Throw error on object clone.
		 *
		 * Cloning instances of the class is forbidden.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @return	void
		 */
		public function __clone() {
			_doing_it_wrong( __FUNCTION__, __( 'You are not allowed to clone this class.', 'uniwin-fortnox' ), '1.0.0' );
		}

		/**
		 * Disable unserializing of the class.
		 *
		 * @access	public
		 * @since	1.0.0
		 * @return	void
		 */
		public function __wakeup() {
			_doing_it_wrong( __FUNCTION__, __( 'You are not allowed to unserialize this class.', 'uniwin-fortnox' ), '1.0.0' );
		}

		/**
		 * Main Uniwin_Fortnox Instance.
		 *
		 * Insures that only one instance of Uniwin_Fortnox exists in memory at any one
		 * time. Also prevents needing to define globals all over the place.
		 *
		 * @access		public
		 * @since		1.0.0
		 * @static
		 * @return		object|Uniwin_Fortnox	The one true Uniwin_Fortnox
		 */
		public static function instance() {
			if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Uniwin_Fortnox ) ) {
				self::$instance					= new Uniwin_Fortnox;
				self::$instance->base_hooks();
				self::$instance->includes();
				self::$instance->helpers		= new Uniwin_Fortnox_Helpers();
				self::$instance->settings		= new Uniwin_Fortnox_Settings();
				self::$instance->ufadminnotice		= new UfAdminNotice();

				//Fire the plugin logic
				new Uniwin_Fortnox_Run();

				/**
				 * Fire a custom action to allow dependencies
				 * after the successful plugin setup
				 */
				do_action( 'UNIWINFORT/plugin_loaded' );
				UfAdminNotice::init();
			}

			return self::$instance;
		}

		/**
		 * Include required files.
		 *
		 * @access  private
		 * @since   1.0.0
		 * @return  void
		 */
		private function includes() {
			require_once UNIWINFORT_PLUGIN_DIR . 'core/includes/classes/class-uniwin-fortnox-helpers.php';
			require_once UNIWINFORT_PLUGIN_DIR . 'core/includes/classes/class-uniwin-fortnox-settings.php';

			require_once UNIWINFORT_PLUGIN_DIR . 'core/includes/classes/class-uniwin-fortnox-run.php';
			require_once UNIWINFORT_PLUGIN_DIR . 'core/includes/classes/class-admin-notice.php';
		}

		/**
		 * Add base hooks for the core functionality
		 *
		 * @access  private
		 * @since   1.0.0
		 * @return  void
		 */
		private function base_hooks() {
			add_action( 'plugins_loaded', array( self::$instance, 'load_textdomain' ) );
		}

		/**
		 * Loads the plugin language files.
		 *
		 * @access  public
		 * @since   1.0.0
		 * @return  void
		 */
		public function load_textdomain() {
			load_plugin_textdomain( 'uniwin-fortnox', FALSE, dirname( plugin_basename( UNIWINFORT_PLUGIN_FILE ) ) . '/languages/' );
		}

	}

endif; // End if class_exists check.