<?php

// Exit if accessed directly.
if (!defined('ABSPATH')) exit;

/**
 * Class Uniwin_Fortnox_Helpers
 *
 * This class contains repetitive functions that
 * are used globally within the plugin.
 *
 * @package		UNIWINFORT
 * @subpackage	Classes/Uniwin_Fortnox_Helpers
 * @author		Mohan
 * @since		1.0.0
 */
class Uniwin_Fortnox_Helpers
{


	/**
	 * ######################
	 * ###
	 * #### CALLABLE FUNCTIONS
	 * ###
	 * ######################
	 */

	/**
	 * HELPER COMMENT START
	 *
	 * Within this class, you can define common functions that you are 
	 * going to use throughout the whole plugin. 
	 * 
	 * Down below you will find a demo function called output_text()
	 * To access this function from any other class, you can call it as followed:
	 * 
	 * UNIWINFORT()->helpers->output_text( 'my text' );
	 * 
	 */

	/**
	 * This function is used to generate an authentication code
	 * */

	public static function getAuthCode()
	{
		$callbackurl = admin_url() . 'admin.php?page=uniwin-fortnox';
		$authorization_redirect_url = FORTNOX_OAUTH_URL . "?response_type=code&client_id=" . FORTNOX_CLIENT_ID . "&redirect_uri=" . urlencode($callbackurl) . "&scope=" . urlencode(FORTNOX_SCOPE) . "&state=fortnox";
		return $authorization_redirect_url;
	}

	/**
	 * This function is used to generate an refresh token
	 * */

	public static function getRefreshToken($auth_code)
	{
		$callbackurl = admin_url() . 'admin.php?page=uniwin-fortnox';
		$authorization = base64_encode(FORTNOX_CLIENT_ID . ":" . FORTNOX_CLIENT_SECRET);
		$url  = 'https://apps.fortnox.se/oauth-v1/token';
		$body = array(
			'grant_type' => 'authorization_code',
			'code' => $auth_code,
			'redirect_uri' => $callbackurl
		);
		$args = array(
			'method'      => 'POST',
			'timeout'     => 45,
			'headers'     => array(
				'Authorization' => "Basic {$authorization}",
				'Content-Type'  => 'application/x-www-form-urlencoded',
			),
			'body'        => $body,
		);
		$request = wp_remote_post($url, $args);
		$response = wp_remote_retrieve_body($request);
		$response_object = json_decode($response, TRUE);
		if (!empty($response_object['error'])) {
			return ['error' => $response_object['error']];
		} else {
			return $response_object;
		}
	}

	/**
	 * This function is used to get an access token
	 * */

	public static function get_access_token()
	{
		$access_token = get_transient('uniwin_fortnox_accesstoken');
		$refresh_token = get_option('uniwin_fortnox_refreshtoken');
		if (!$access_token) {
			$authorization = base64_encode(FORTNOX_CLIENT_ID . ":" . FORTNOX_CLIENT_SECRET);
			$url  = 'https://apps.fortnox.se/oauth-v1/token';
			$body = array(
				'grant_type' => 'refresh_token',
				'refresh_token' => $refresh_token,
			);
			$args = array(
				'method'      => 'POST',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Basic {$authorization}",
					'Content-Type'  => 'application/x-www-form-urlencoded',
				),
				'body'        => $body,
			);
			$request = wp_remote_post($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);

			if (!empty($response_object['error'])) {
				//header('Location:' . admin_url() . 'admin.php?page=uniwin-fortnox&error_description=' . $response_object['error_description']);
				return null;
			} else {
				$access_token = $response_object['access_token'];
				update_option('uniwin_fortnox_accesstoken', $access_token);
				update_option('uniwin_fortnox_refreshtoken', $response_object['refresh_token']);
				set_transient('uniwin_fortnox_accesstoken', $response_object['access_token'], $response_object['expires_in'] - 300);
			}
		}
		return $access_token;
	}

	/**
	 * This function is used to get an all fortnox projects
	 * */

	public static function get_fortnox_projects()
	{
		$access_token = self::get_access_token();
		$projects = [];
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/projects';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting projects'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * This function is used to get an all fortnox units
	 **/

	public static function get_fortnox_units()
	{
		$access_token = self::get_access_token();
		$units = [];
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/units';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting units'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * This function is used to get an all fortnox accounts
	 **/

	public static function get_fortnox_accounts()
	{
		$access_token = self::get_access_token();
		$accounts = [];
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/accounts';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting Accounts'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * This function is used to get an all fortnox deliveryterms
	 **/

	public static function get_fortnox_deliveryterms()
	{
		$access_token = self::get_access_token();
		$termsofdeliveries = [];
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/termsofdeliveries';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting TermsOfDeliveries'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * This function is used to get an all wayofdeliveries
	 **/

	public static function get_fortnox_wayofdeliveries()
	{
		$access_token = self::get_access_token();
		$wayofdeliveries = [];
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/wayofdeliveries';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting WayOfDeliveries'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
		return $wayofdeliveries;
	}


	/**
	 * This function is used to get an all costcenters
	 **/

	public static function get_fortnox_costcenters()
	{
		$access_token = self::get_access_token();
		$costcenters = [];
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/costcenters';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting CostCenters'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
		return $costcenters;
	}

	/**
	 * This function is used to get an all customers
	 **/

	public static function get_fortnox_customers($data)
	{
		$access_token = self::get_access_token();
		$customers = [];
		$params = (!empty($data)) ? '?' . http_build_query($data) : '';
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/customers' . $params;
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error getting Customers'];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
		return $customers;
	}

	/**
	 * This function is used to add customer to fortnox
	 **/

	public static function add_fortnox_customer($data)
	{
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/customers';
			$args = array(
				'method'      => 'POST',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($data),
			);
			$request = wp_remote_post($url, $args);
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => 'Error adding customer. ' . $response_object['ErrorInformation']];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Generate fortnox api data for order and invoice
	 **/

	public static function fortnox_data($order_id, $type)
	{
		$order = wc_get_order($order_id);
		$uf_accountssettings = json_decode(json_encode(get_option('uf_accountssettings')));
		$uf_ordersettings = json_decode(json_encode(get_option('uf_ordersettings')));
		$uf_shippingsettings = json_decode(json_encode(get_option('uf_shippingsettings')));
		$order_data  = $order->get_data();
		$customer_name = $order_data['billing']['first_name'] . ' ' . $order_data['billing']['last_name'];
		$DeliveryName = $order_data['shipping']['first_name'] . ' ' . $order_data['shipping']['last_name'];
		$remarks = "Wordpress order number : " . $order_data['id'];
		$country = (!empty($order_data['billing']['country'])) ? WC()->countries->countries[$order_data['billing']['country']] : '';
		if ($country == 'Sweden') {
			$country = 'Sverige';
		}

		$customer_number = "";
		$customers = self::get_fortnox_customers(['email' => $order_data['billing']['email']]);
		if (!empty($customers)) {
			$customer_number = $customers['Customers'][0]['CustomerNumber'];
		}
		if ($customer_number == "") {
			$customer_data = array(
				"Customer" => array(
					"Email" => $order_data['billing']['email'],
					'Name' => $customer_name,
					"Address1" => (!empty($order_data['billing']['address_1'])) ? $order_data['billing']['address_1'] : '',
					"Address2" => (!empty($order_data['billing']['address_2'])) ? $order_data['billing']['address_2'] : '',
					"City" => (!empty($order_data['billing']['city'])) ? $order_data['billing']['city'] : '',
					"CountryCode" => (!empty($order_data['billing']['country'])) ? $order_data['billing']['country'] : '',
					"ZipCode" => (!empty($order_data['billing']['postcode'])) ? $order_data['billing']['postcode'] : '',
					"DeliveryAddress1" => (!empty($order_data['shipping']['address_1'])) ? $order_data['shipping']['address_1'] : '',
					"DeliveryAddress2" => (!empty($order_data['shipping']['address_2'])) ? $order_data['shipping']['address_2'] : '',
					"DeliveryCity" => (!empty($order_data['shipping']['city'])) ? $order_data['shipping']['city'] : '',
					"DeliveryCountryCode" => (!empty($order_data['shipping']['country'])) ? $order_data['shipping']['country'] : '',
					"DeliveryZipCode" => (!empty($order_data['shipping']['postcode'])) ? $order_data['shipping']['postcode'] : '',
					"ShowPriceVATIncluded" => false,
					'Type' => 'PRIVATE',

				)
			);
			$customer = self::add_fortnox_customer($customer_data);
			$customer_number = (!empty($customer['Customer']['CustomerNumber'])) ? $customer['Customer']['CustomerNumber'] : '1';
		}
		$line_items = [];
		foreach ($order->get_items() as $item) {
			$product = $item->get_product(); // Get the WC_Product Object
			$tax_class = $item->get_tax_class('edit');
			$tax_rates = WC_Tax::get_base_tax_rates($tax_class);
			if (count($tax_rates)) {
				$tax_rate = reset($tax_rates);
				$tax_rate = $tax_rate['rate'];
			} else {
				$tax_rate = '0';
			}
			if ($order_data['billing']['country'] == 'SE') {
				$option = 'uf_account_' . $tax_rate . '_vat';
				$account = $uf_accountssettings->$option ?? '';
			} else if (self::isEU($order_data['billing']['country'])) {
				$option = 'uf_account_eu_inc_vat';
				$account = $uf_accountssettings->$option;
			} else {
				$option = 'uf_account_other_countries';
				$account = $uf_accountssettings->$option;
			}

			$article_number = '';
			if (!empty($product->get_sku())) {
				$article = self::get_fortnox_article($product->get_sku());
				if (!empty($article['Article']['ArticleNumber']) && $article['Article']['Active'] == true) {
					$article_number = $article['Article']['ArticleNumber'];
				}
			}
			if ($article_number == '') {
				$fortnox_product = self::createArticle($product->get_id());
				$article_number = $fortnox_product['Article']['ArticleNumber'];
			}
			$include_tax = (isset($uf_ordersettings->excl_tax) && $uf_ordersettings->excl_tax == '1') ? false : true;
			$line_items[] = array(
				'ArticleNumber' => (!empty($article_number)) ? $article_number : '',
				"AccountNumber" => (!empty($account)) ? $account : '',
				"DeliveredQuantity" => $item->get_quantity(),
				"Description" => $product->get_name(),
				"Price" =>  $product->get_price(),
				"VAT" => $tax_rate,
			);
		}

		if ($type == 'Order') {
			$order_date = date('Y-m-d', strtotime($order->order_date));
			foreach ($order->get_items('shipping') as $shipping_item_id => $shipping_item) {
				$order_item_name             = $item->get_name();
				$order_item_type             = $item->get_type();
				$shipping_method_title       = $shipping_item->get_method_title();
				$shipping_method_id          = $shipping_item->get_method_id(); // The method ID
				$shipping_method_instance_id = $shipping_item->get_instance_id(); // The instance ID
				$shipping_method_total       = $shipping_item->get_total();
				$shipping_method_total_tax   = $shipping_item->get_total_tax();
				$shipping_method_taxes       = $shipping_item->get_taxes();

				$uf_shippingarticle_id = 'uf_shippingarticle_' . $shipping_method_id . '_' . $shipping_method_instance_id;
				$uf_deliveryterms_id = 'uf_deliveryterms_' . $shipping_method_id . '_' . $shipping_method_instance_id;
				$uf_deliverymethods_id = 'uf_deliverymethod_' . $shipping_method_id . '_' . $shipping_method_instance_id;
				$article_number_1 = (isset($uf_shippingsettings->$uf_shippingarticle_id) && !empty($uf_shippingsettings->$uf_shippingarticle_id)) ? $uf_shippingsettings->$uf_shippingarticle_id : 'UF_BLANK';

				$shipping_article_number = '';
				if (!empty($article_number_1)) {
					$article = self::get_fortnox_article($article_number_1);
					if (!empty($article['Article']['ArticleNumber']) && $article['Article']['Active'] == true) {
						$shipping_article_number = $article['Article']['ArticleNumber'];
					}
				}

				$line_items[] = array(
					'ArticleNumber' => (!empty($shipping_article_number)) ? $shipping_article_number : '',
					"DeliveredQuantity" => '1',
					"Description" => $shipping_method_title,
					"Price" =>  $shipping_method_total,
					"VAT" => $shipping_method_total_tax ?? '',
				);
				$deliveryterms = (isset($uf_shippingsettings->$uf_deliveryterms_id) && !empty($uf_shippingsettings->$uf_deliveryterms_id)) ? $uf_shippingsettings->$uf_deliveryterms_id : '';
				$deliveryways = (isset($uf_shippingsettings->$uf_deliverymethods_id) && !empty($uf_shippingsettings->$uf_deliverymethods_id)) ? $uf_shippingsettings->$uf_deliverymethods_id : '';
			}

			$fortnox_data = array(
				"Order" => array(
					"Address1" => (!empty($order_data['billing']['address_1'])) ? $order_data['billing']['address_1'] : '',
					"Address2" => (!empty($order_data['billing']['address_2'])) ? $order_data['billing']['address_2'] : '',
					"City" => (!empty($order_data['billing']['city'])) ? $order_data['billing']['city'] : '',
					"Country" => $country,
					"ZipCode" => (!empty($order_data['billing']['postcode'])) ? $order_data['billing']['postcode'] : '',
					"CustomerNumber" => $customer_number,
					"CustomerName" =>  $customer_name,
					"DeliveryName" => $DeliveryName,
					"DeliveryAddress1" => (!empty($order_data['shipping']['address_1'])) ? $order_data['shipping']['address_1'] : '',
					"DeliveryAddress2" => (!empty($order_data['shipping']['address_2'])) ? $order_data['shipping']['address_2'] : '',
					"DeliveryCity" => (!empty($order_data['shipping']['city'])) ? $order_data['shipping']['city'] : '',
					"DeliveryCountry" => $country,
					"DeliveryZipCode" => (!empty($order_data['shipping']['postcode'])) ? $order_data['shipping']['postcode'] : '',
					"Remarks" => $remarks,
					"OrderRows" => $line_items,
					"Currency" => $order->get_currency(),
					"OrderDate" => $order_date,
					"YourOrderNumber" => $order->get_id(),
					"YourReference" => null,
					"Freight" => '',
					'OurReference' => (!empty($uf_ordersettings->our_reference)) ? $uf_ordersettings->our_reference : '',
					'VATIncluded' => (isset($uf_ordersettings->excl_tax) && $uf_ordersettings->excl_tax == '1') ? false : true,
					'DeliveryDate' => (isset($uf_ordersettings->delivery_days) && $uf_ordersettings->delivery_days != '') ? date('Y-m-d', strtotime($order_date . ' + ' . $uf_ordersettings->delivery_days . ' days')) : '',
					'Project' => (isset($uf_ordersettings->project) && $uf_ordersettings->project != '') ? $uf_ordersettings->project : '',
					'CostCenter' => (isset($uf_ordersettings->cost_center) && $uf_ordersettings->cost_center != '') ? $uf_ordersettings->cost_center : '',
					'TermsOfDelivery' => (!empty($deliveryterms)) ? $deliveryterms : '',
					'WayOfDelivery' => (!empty($deliveryways)) ? $deliveryways : '',
				)
			);
		}

		if ($type == 'Invoice') {
			$fortnox_data = array(
				"Invoice" => array(
					"Address1" => (!empty($order_data['billing']['address_1'])) ? $order_data['billing']['address_1'] : '',
					"Address2" => (!empty($order_data['billing']['address_2'])) ? $order_data['billing']['address_2'] : '',
					"City" => (!empty($order_data['billing']['city'])) ? $order_data['billing']['city'] : '',
					"Country" => $country,
					"ZipCode" => (!empty($order_data['billing']['postcode'])) ? $order_data['billing']['postcode'] : '',
					"CustomerNumber" => $customer_number,
					"CustomerName" =>  $customer_name,
					"DeliveryName" => $DeliveryName,
					"DeliveryAddress1" => (!empty($order_data['shipping']['address_1'])) ? $order_data['shipping']['address_1'] : '',
					"DeliveryAddress2" => (!empty($order_data['shipping']['address_2'])) ? $order_data['shipping']['address_2'] : '',
					"DeliveryCity" => (!empty($order_data['shipping']['city'])) ? $order_data['shipping']['city'] : '',
					"DeliveryCountry" => $country,
					"DeliveryZipCode" => (!empty($order_data['shipping']['postcode'])) ? $order_data['shipping']['postcode'] : '',
					"Remarks" => $remarks,
					"InvoiceRows" => $line_items,
					"Currency" => $order->get_currency(),
					"YourOrderNumber" => $order->get_id(),
					"Freight" => 20
				)
			);
		}

		return $fortnox_data;
	}


	/**
	 * Generate fortnox api data for article
	 **/

	public static function fortnox_article_data($product, $parent_id = null)
	{
		// get parent product if product is variation
		if ($parent_id) {
			$parentproduct = wc_get_product($parent_id);
			$PurchasePrice = get_post_meta($parentproduct->get_id(), '_uf_purchase_price', true);
			$EAN = get_post_meta($parentproduct->get_id(), '_uf_ean', true);
			$uf_manufacturer = get_post_meta($parentproduct->get_id(), '_uf_manufacturer', true);
			$uf_unit = get_post_meta($parentproduct->get_id(), '_uf_product_unit', true);
			$uf_manufacturer_article = get_post_meta($parentproduct->get_id(), '_uf_manufacturer_article', true);
		} else {
			$PurchasePrice = get_post_meta($product->get_id(), '_uf_purchase_price', true);
			$EAN = get_post_meta($product->get_id(), '_uf_ean', true);
			$uf_manufacturer = get_post_meta($product->get_id(), '_uf_manufacturer', true);
			$uf_unit = get_post_meta($product->get_id(), '_uf_product_unit', true);
			$uf_manufacturer_article = get_post_meta($product->get_id(), '_uf_manufacturer_article', true);
		}

		$with_tax = wc_get_price_including_tax($product);
		$without_tax = wc_get_price_excluding_tax($product);
		$tax_amount = $with_tax - $without_tax;
		$vat_percent = ($without_tax && $without_tax != 0) ? ($tax_amount / $without_tax) * 100 : '';
		$fortnox_article_data = array(
			"Article" => array(
				"ArticleNumber" => $product->get_sku() ?? null,
				"EAN" => (!empty($EAN)) ? $EAN : '',
				"Bulky" => false,
				"Description" => $product->get_name(),
				"Expired" => false,
				"Housework" => false,
				"Note" => "",
				"PurchasePrice" => (!empty($PurchasePrice)) ? $PurchasePrice : '',
				"VAT" => $vat_percent,
				"StockPlace" => null,
				"WebshopArticle" => false,
				"Active" => true,
				"Manufacturer" => (!empty($uf_manufacturer)) ? $uf_manufacturer : '',
				'Unit' => (!empty($uf_unit)) ? $uf_unit : '',
				"ManufacturerArticleNumber" => (!empty($uf_manufacturer_article)) ? $uf_manufacturer_article : '',
				"QuantityInStock" => $product->get_stock_quantity(),
				"Weight" => (!empty($product->get_weight())) ? $product->get_weight() * 1000 : '',
				"Width" => (!empty($product->get_width())) ? $product->get_width() * 10 : '',
				"Height" => (!empty($product->get_height())) ? $product->get_height() * 10 : '',
				"Depth" => (!empty($product->get_length())) ? $product->get_length() * 10 : '',
			)
		);
		return $fortnox_article_data;
	}

	/**
	 * Sync order function
	 **/


	public static function syncorder($order_id)
	{
		$order = wc_get_order($order_id);
		$fortnox_data = self::fortnox_data($order_id, 'Order');
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/orders';
			$args = array(
				'method'      => 'POST',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($fortnox_data),
			);
			$request = wp_remote_post($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Uniwin Fortnox Order sync Failed'];
			} else {
				$response = wp_remote_retrieve_body($request);
				$response_object = json_decode($response, TRUE);
				if (!isset($response_object['ErrorInformation'])) {
					return $response_object;
				} else {
					return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when processing order #' . $order_id : 'Order sync error when processing order #' . $order_id];
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * ReSync order function
	 **/

	public static function order_resync($order_id)
	{
		$fortnox_data = self::fortnox_data($order_id, 'Order');
		$fortnox_order_number = get_post_meta($order_id, '_uf_order_number', true);
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/orders/' . $fortnox_order_number;
			$args = array(
				'method'      => 'PUT',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($fortnox_data),
			);
			$request = wp_remote_post($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Uniwin Fortnox Order Resync Failed'];
			} else {
				$response = wp_remote_retrieve_body($request);
				$response_object = json_decode($response, TRUE);
				if (!isset($response_object['ErrorInformation'])) {
					return $response_object;
				} else {
					return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when processing order #' . $order_id : 'Order resync error  when processing order #' . $order_id];
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Create Invoice function
	 **/

	public static function create_invoice($order_id)
	{
		$fortnox_order_number = get_post_meta($order_id, '_uf_order_number', true);
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/orders/' . $fortnox_order_number . '/createinvoice';
			$args = array(
				'method'      => 'PUT',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_post($url, $args);

			if (is_wp_error($request)) {
				return ['error' => 'Uniwin Fortnox Create Invoice Failed'];
			} else {
				$response = wp_remote_retrieve_body($request);
				$response_object = json_decode($response, TRUE);
				if (!isset($response_object['ErrorInformation'])) {
					return $response_object;
				} else {
					return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when processing order #' . $order_id : 'Order invoice error when processing order #' . $order_id];
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Get Invoice function
	 **/

	public static function get_invoice($fortnox_invoice_number)
	{
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/invoices/' . $fortnox_invoice_number;
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Get Invoice Failed'];
			} else {
				$response = wp_remote_retrieve_body($request);
				$response_object = json_decode($response, TRUE);
				if (!isset($response_object['ErrorInformation'])) {
					return $response_object;
				} else {
					return ['error' => 'Get Invoice Failed'];
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}


	/**
	 * Send Invoice function
	 **/

	public static function send_invoice($order_id)
	{
		$order = wc_get_order($order_id);
		$fortnox_invoice_number = get_post_meta($order_id, '_uf_invoice_number', true);
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/invoices/' . $fortnox_invoice_number . '/email';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_get($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Send invoice error'];
			}
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when processing order #' . $order_id : 'Order send invoice error when processing order #' . $order_id];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Book Invoice function
	 **/
	public static function book_invoice($order_id)
	{
		$order = wc_get_order($order_id);
		$fortnox_invoice_number = get_post_meta($order_id, '_uf_invoice_number', true);
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/invoices/' . $fortnox_invoice_number . '/bookkeep';
			$args = array(
				'method'      => 'PUT',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_post($url, $args);

			if (is_wp_error($request)) {
				return ['error' => 'Book invoice error'];
			}
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when processing order #' . $order_id : 'Order book invoice error when processing order #' . $order_id];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Print Invoice function
	 **/

	public static function print_invoice($order_id)
	{
		$fortnox_invoice_number = get_post_meta($order_id, '_uf_invoice_number', true);
		$access_token = self::get_access_token();
		if ($access_token) {
			$url  = FORTNOX_API_URL . '/3/invoices/' . $fortnox_invoice_number . '/print';
			$args = array(
				'method'      => 'GET',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
			);
			$request = wp_remote_post($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Invoice payment Error'];
			}
			$response = wp_remote_retrieve_body($request);
			if (preg_match("/^%PDF-1.5/", $response)) {
				return chunk_split(base64_encode($response));
			} else {
				return ['error' => 'Error Printing order #' . $order_id];
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Update invoice payment
	 **/

	public static function update_invoice_payment($order_id)
	{
		$order = wc_get_order($order_id);
		$access_token = self::get_access_token();
		if ($access_token) {
			$fortnox_invoice_number = get_post_meta($order_id, '_uf_invoice_number', true);
			$url  = FORTNOX_API_URL . '/3/invoicepayments/';
			$invoice_data = array(
				'InvoicePayment' => array(
					'Amount' => doubleval(($order->get_total() - $order->get_total_refunded())),
					'InvoiceNumber' => $fortnox_invoice_number,
				)
			);
			$args = array(
				'method'      => 'POST',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($invoice_data),
			);
			$request = wp_remote_post($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Invoice payment Error'];
			}
			$response = wp_remote_retrieve_body($request);
			$response_object = json_decode($response, TRUE);
			if (!empty($response_object['ErrorInformation'])) {
				return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when processing order #' . $order_id : 'Invoice payment error when processing order #' . $order_id];
			} else {
				return $response_object;
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	/**
	 * Create article
	 **/

	public static function createArticle($product_id, $parent_id = null)
	{
		$access_token = self::get_access_token();
		if ($access_token) {
			$product = wc_get_product($product_id);
			$uf_article_number = get_post_meta($product_id, '_uf_article_number', true);
			$fortnox_article_data = self::fortnox_article_data($product, $parent_id);
			$url  = FORTNOX_API_URL . '/3/articles/';
			$args = array(
				'method'      => "POST",
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($fortnox_article_data),
			);
			$request = wp_remote_post($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Error creating article #' . $product_id];
			} else {
				$response = wp_remote_retrieve_body($request);
				$response_object = json_decode($response, TRUE);
				if (!empty($response_object['Article'])) {
					// add price of article
					self::addPrice($response_object['Article']['ArticleNumber'], $product->get_price(), $access_token);
					return $response_object;
				} else {
					return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when creating article #' . $product_id : 'Error creating article #' . $product_id];
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}

	public static function addPrice($articleNumber, $price, $access_token)
	{
		$priceList = "A";
		$data = array(
			"ArticleNumber" => $articleNumber,
			"FromQuantity" => 0,
			"Price" => $price,
			"PriceList" => $priceList,
			"PriceList" => "A"
		);
		$articleNumber = urlencode($articleNumber);
		//$ret = $this->api('GET', "prices/{$priceList}/{$articleNumber}/0", $data, $access_token);

		// wordpress code for above line
		$url = FORTNOX_API_URL . '/3/prices/' . $priceList . '/' . $articleNumber . '/0';
		$args = array(
			'method'      => 'GET',
			'timeout'     => 45,
			'headers'     => array(
				'Authorization' => "Bearer {$access_token}",
				'Content-Type'  => 'application/json',
			),
		);
		$request = wp_remote_get($url, $args);
		if (is_wp_error($request)) {
			return ['error' => 'Error getting price'];
		} else {
			$response = wp_remote_retrieve_body($request);
			$arrRet = json_decode($response, TRUE);
			if (!empty($arrRet['ErrorInformation'])) {
				$isPriceListExist = false;
			} else {
				$isPriceListExist = true;
			}
		}
		if ($isPriceListExist) {
			//$response = $this->api('PUT', "prices/{$priceList}/{$articleNumber}/0", array("Price" => $data), $access_token);
			//wordpress code for above line
			$url = FORTNOX_API_URL . '/3/prices/' . $priceList . '/' . $articleNumber . '/0';

			$args = array(
				'method'      => 'PUT',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode(array("Price" => $data)),
			);
		} else {
			$url = FORTNOX_API_URL . '/3/prices/' . $priceList . '/' . $articleNumber;
			$args = array(
				'method'      => 'POST',
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($data),
			);
		}
		$result = wp_remote_request($url, $args);
		if (200 === wp_remote_retrieve_response_code($result)) {
			$body = wp_remote_retrieve_body($result);
			$arrResponse = json_decode($body, true);
			if (isset($arrResponse['ErrorInformation'])) {
				return $arrResponse;
			}
			return true;
		} else {
			return ['error' => 'Error updating price'];
		}
	}

	/**
	 * This function is used to update article
	 **/

	public static function updateArticle($product_id, $parent_id = null)
	{
		$access_token = self::get_access_token();
		if ($access_token) {
			$product = wc_get_product($product_id);
			$uf_article_number = get_post_meta($product_id, '_uf_article_number', true);
			$fortnox_article_data = self::fortnox_article_data($product, $parent_id);

			$url  = FORTNOX_API_URL . '/3/articles/' . $uf_article_number;
			$args = array(
				'method'      => "PUT",
				'timeout'     => 45,
				'headers'     => array(
					'Authorization' => "Bearer {$access_token}",
					'Content-Type'  => 'application/json',
				),
				'body' => json_encode($fortnox_article_data),
			);
			$request = wp_remote_post($url, $args);
			if (is_wp_error($request)) {
				return ['error' => 'Error creating article #' . $product_id];
			} else {
				$response = wp_remote_retrieve_body($request);
				$response_object = json_decode($response, TRUE);
				if (!empty($response_object['Article'])) {
					return $response_object;
				} else {
					return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when creating article #' . $product_id : 'Error creating article #' . $product_id];
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}
	/**
	 * This function is used to get article
	 **/

	public static function get_fortnox_article($ArticleNumber)
	{
		$access_token = self::get_access_token();
		if ($access_token) {
			$article = null;
			if ($access_token) {
				$url  = FORTNOX_API_URL . '/3/articles/' . $ArticleNumber;
				$args = array(
					'method'      => 'GET',
					'timeout'     => 45,
					'headers'     => array(
						'Authorization' => "Bearer {$access_token}",
						'Content-Type'  => 'application/json',
					),
				);
				$request = wp_remote_get($url, $args);
				if (is_wp_error($request)) {
					return ['error' => 'Get article failed'];
				} else {
					$response = wp_remote_retrieve_body($request);
					$response_object = json_decode($response, TRUE);
					if (!empty($response_object['Article'])) {
						return $response_object;
					} else {
						return ['error' => (!empty($response_object['ErrorInformation']['message'])) ? $response_object['ErrorInformation']['message'] . ' when creating article #' . $ArticleNumber : 'Error creating article #' . $ArticleNumber];
					}
				}
			}
		} else {
			return ['error' => 'Error connecting fortnox. Please reconnect!'];
		}
	}
	/**
	 * This function checks if the country is EU ?
	 **/
	public static function isEU($countrycode)
	{
		$eu_countrycodes = array(
			'AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'EL',
			'ES', 'FI', 'FR', 'GR', 'HR', 'HU', 'IE', 'IT', 'LT', 'LU', 'LV',
			'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK'
		);
		return (in_array($countrycode, $eu_countrycodes));
	}
}
