<?php

// Exit if accessed directly.
if (!defined('ABSPATH')) exit;

/**
 * HELPER COMMENT START
 * 
 * This class is used to bring your plugin to life. 
 * All the other registered classed bring features which are
 * controlled and managed by this class.
 * 
 * Within the add_hooks() function, you can register all of 
 * your WordPress related actions and filters as followed:
 * 
 * add_action( 'my_action_hook_to_call', array( $this, 'the_action_hook_callback', 10, 1 ) );
 * or
 * add_filter( 'my_filter_hook_to_call', array( $this, 'the_filter_hook_callback', 10, 1 ) );
 * or
 * add_shortcode( 'my_shortcode_tag', array( $this, 'the_shortcode_callback', 10 ) );
 * 
 * Once added, you can create the callback function, within this class, as followed: 
 * 
 * public function the_action_hook_callback( $some_variable ){}
 * or
 * public function the_filter_hook_callback( $some_variable ){}
 * or
 * public function the_shortcode_callback( $attributes = array(), $content = '' ){}
 * 
 * 
 * HELPER COMMENT END
 */

/**
 * Class Uniwin_Fortnox_Run
 *
 * Thats where we bring the plugin to life
 *
 * @package		UNIWINFORT
 * @subpackage	Classes/Uniwin_Fortnox_Run
 * @author		Mohan
 * @since		1.0.0
 */
class Uniwin_Fortnox_Run
{

	public $uf_productsettings;
	public $uf_ordersettings;
	public $uf_invoicesettings;
	public $uf_shippingsettings;
	public $uf_accountssettings;

	/**
	 * Our Uniwin_Fortnox_Run constructor 
	 * to run the plugin logic.
	 *
	 * @since 1.0.0
	 */
	function __construct()
	{
		$this->uf_productsettings = json_decode(json_encode(get_option('uf_productsettings')));
		$this->uf_ordersettings = json_decode(json_encode(get_option('uf_ordersettings')));
		$this->uf_invoicesettings = json_decode(json_encode(get_option('uf_invoicesettings')));
		$this->uf_shippingsettings = json_decode(json_encode(get_option('uf_shippingsettings')));
		$this->uf_accountssettings = json_decode(json_encode(get_option('uf_accountssettings')));

		$this->add_hooks();
	}

	/**
	 * ######################
	 * ###
	 * #### WORDPRESS HOOKS
	 * ###
	 * ######################
	 */

	/**
	 * Registers all WordPress and plugin related hooks
	 *
	 * @access	private
	 * @since	1.0.0
	 * @return	void
	 */
	private function add_hooks()
	{

		add_action('admin_menu', array($this, 'menu_so_17406309'));
		add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'plugin_action_links'));
		add_filter('manage_shop_order_posts_custom_column', array($this, 'custom_orders_list_column_content'), 20, 2);
		add_filter('manage_edit-shop_order_columns', array($this, 'custom_shop_order_column'), 20);
		add_action('woocommerce_checkout_order_processed', array($this, 'order_payment_complete'));
		add_action('woocommerce_order_status_completed', array($this, 'order_payment_complete'));
		add_action('woocommerce_order_status_processing', array($this, 'order_payment_complete'));
		add_action('woocommerce_order_status_pending', array($this, 'order_payment_complete'));
		add_action('wp_ajax_order_resync', array($this, 'order_resync'));
		add_action('wp_ajax_ajx_order_sync', array($this, 'ajx_order_sync'));
		add_action('wp_ajax_ajx_order_resync', array($this, 'ajx_order_resync'));
		add_action('add_meta_boxes', array($this, 'uniwin_order_metabox'));
		add_action('save_post', array($this,  'uniwin_order_metabox_save'), 10, 1);

		add_action('woocommerce_new_product', array($this,  'uf_on_product_save'), 10, 1);
		add_action('woocommerce_update_product', array($this,  'uf_on_product_save'), 10, 1);

		add_action('woocommerce_admin_process_product_object', array($this, 'uf_save_product_customfields'));

		add_filter('cron_schedules', array($this, 'uwfx_cron_schedules'));
		add_action('uwfx_autoinvoice_hook', array($this, 'uwfx_schedule_invoice_function'), 10, 2);
		add_action('wp', array($this, 'schedule_invoice_task'));

		add_action('admin_enqueue_scripts', array($this, 'enqueue_backend_scripts_and_styles'), 20);
		add_action('plugins_loaded', array($this, 'add_wp_webhooks_integrations'), 9);
	}

	/**
	 * ######################
	 * ###
	 * #### WORDPRESS HOOK CALLBACKS
	 * ###
	 * ######################
	 */

	public function plugin_action_links($links)
	{
		$settings_link = '<a href="' . esc_url(admin_url('/admin.php?page=uniwin-fortnox')) . '">' . __('Settings', '_uf') . '</a>';
		array_unshift($links, $settings_link);
		return $links;
	}
	/**
	 * Add options page
	 */
	public function menu_so_17406309()
	{
		add_menu_page(
			'Uniwin Fortnox',
			'Uniwin Fortnox',
			'manage_options',
			'uniwin-fortnox',
			array($this, 'callback_so_17406309'),
			$icon_url = ''
		);
	}

	public function callback_so_17406309()
	{
		include_once UNIWINFORT_PLUGIN_DIR . 'views/index.php';
	}

	/**
	 * Auto sync webhook callback function
	 **/

	public function order_payment_complete($order_id)
	{
		$order = wc_get_order($order_id);
		$uf_auto_sync_order = get_post_meta($order_id, '_uf_auto_sync_order', true);
		if (!empty($uf_auto_sync_order) && $uf_auto_sync_order == '1') {
			return false;
		}
		$order_status  = $order->get_status();
		$order_sync = false;
		if (isset($this->uf_ordersettings->autosync) && $this->uf_ordersettings->autosync == '1') {
			$order_sync = true;
			if (!empty($this->uf_ordersettings->autosync_status) && $this->uf_ordersettings->autosync_status != $order_status) {
				$order_sync = false;
			}
		}
		if ($order_sync) {
			$syncorder = Uniwin_Fortnox_Helpers::syncorder($order_id);
			if (!isset($syncorder['error'])) {
				$order->update_meta_data('_uf_auto_sync_order', '1');
				$order->update_meta_data('_uf_order_number', $syncorder['Order']['DocumentNumber']);
				$order->update_meta_data('_uf_order_url', $syncorder['Order']['@url']);
				$order->save();

				UfAdminNotice::error('Uniwin Fortnox : Order Sync Success Order id: #' . $order_id, true);
				if (isset($this->uf_invoicesettings->create_invoice) && $this->uf_invoicesettings->create_invoice == '1') {
					$create_invoice = Uniwin_Fortnox_Helpers::create_invoice($order_id);
					if (!isset($create_invoice['error'])) {
						$order->update_meta_data('_uf_invoice_number', $create_invoice['Order']['InvoiceReference']);
						$order->save();
						if (isset($this->uf_invoicesettings->send_invoice) && $this->uf_invoicesettings->send_invoice == '1') {
							Uniwin_Fortnox_Helpers::send_invoice($order_id);
							$order->update_meta_data('_uf_invoice_sent', '1');
							$order->save();
						}
						if (isset($this->uf_invoicesettings->book_invoice) && $this->uf_invoicesettings->book_invoice == '1') {
							$book_invoice = Uniwin_Fortnox_Helpers::book_invoice($order_id);
							if (!isset($book_invoice['error']) && $order->get_payment_method() == 'cod') {
								$order->update_meta_data('_uf_invoice_booked', '1');
								$order->save();
								$order->payment_complete();
								$note = 'Fortnox Invoice ' . $create_invoice['Order']['InvoiceReference'] . ' booked. Order status updated to paid';
								$order->add_order_note($note);
							}
						}
						if (isset($this->uf_invoicesettings->add_invoicepayment) && $this->uf_invoicesettings->add_invoicepayment == '1' && $order->is_paid()) {
							$add_payment = Uniwin_Fortnox_Helpers::update_invoice_payment($order_id);
							if (!isset($add_payment['error'])) {
								$order->update_meta_data('_uf_invoice_paid', '1');
								$order->save();
								$this->uf_log_action('Update invoice payment: ' . $create_invoice['Order']['InvoiceReference']);
							} else {
								$this->uf_log_action($add_payment['error']);
							}
						}
					} else {
						$this->uf_log_action($create_invoice['error']);
					}
				}
			} else {
				UfAdminNotice::error('Uniwin Fortnox : ' . $syncorder['error'], true);
			}
		}
	}

	/**
	 * Creates custom columns in woocommerce orders table
	 **/

	public  function custom_shop_order_column($columns)
	{
		$reordered_columns = array();
		foreach ($columns as $key => $column) {
			$reordered_columns[$key] = $column;
			if ($key ==  'order_status') {
				$reordered_columns['resync'] = __('Fortnox Resync', 'uniwin');
				$reordered_columns['_uforder_number'] = __('Fortnox Order ID', 'uniwin');
			}
		}
		return $reordered_columns;
	}

	public function custom_orders_list_column_content($column, $post_id)
	{
		switch ($column) {
			case 'resync':
				$fortnox_order_number = get_post_meta($post_id, '_uf_order_number', true);
				if (!empty($fortnox_order_number))
					echo '<a class="button button-primary uf-resync" href="javascript:;" data-id="' . $post_id . '">Resync</a>';
				else
					echo '<a class="button button-primary uf-sync" data-id="' . $post_id . '">Sync</a>';
				break;
			case '_uforder_number':
				$fortnoxorder_number = get_post_meta($post_id, '_uf_order_number', true);
				if (!empty($fortnoxorder_number))
					echo $fortnoxorder_number;
				else
					echo '<small>-</small>';
				break;
		}
	}

	/**
	 * Ajax order sync function
	 **/

	public function ajx_order_sync()
	{
		$order_id = $_POST['order_id'];
		$order = wc_get_order($order_id);
		$create_order = Uniwin_Fortnox_Helpers::syncorder($order_id);
		if (isset($create_order['error'])) {
			UfAdminNotice::error('Uniwin Fortnox : ' . $create_order['error'], true, $order_id);
			echo 'failed';
		} else {
			$order->update_meta_data('_uf_order_number', $create_order['Order']['DocumentNumber']);
			$order->update_meta_data('_uf_order_url', $create_order['Order']['@url']);
			$order->save();
			UfAdminNotice::success("Uniwin Fortnox : Order sync success", true, $order_id);
			echo 'success';
		}
		die();
	}


	/**
	 * Ajax order resync function
	 **/

	public function ajx_order_resync()
	{
		$order_id = $_POST['order_id'];
		$resync = Uniwin_Fortnox_Helpers::order_resync($order_id);
		if (isset($resync['error'])) {
			UfAdminNotice::error('Uniwin Fortnox : ' . $resync['error'], true, $order_id);
			echo 'failer';
		} else {
			UfAdminNotice::success("Uniwin Fortnox : Order sync success", true, $order_id);
			echo 'success';
		}
		die();
	}


	/**
	 * Creates Order detail page meta box
	 **/

	public function uniwin_order_metabox()
	{
		add_meta_box('mv_other_fields', __('Uniwin Fortnox', 'woocommerce'), array($this, 'uniwin_fortnox_order_action'), 'shop_order', 'side', 'core');
		if (!empty($this->uf_productsettings) && !empty($this->uf_productsettings->sync_fields)) {
			add_meta_box('uf_product_fields', __('Uniwin Fortnox', 'woocommerce'), array($this, 'uniwin_fortnox_product_action'), 'product', 'side', 'core');
		}
	}

	/**
	 * Order detail page meta box fields
	 **/

	public function uniwin_fortnox_order_action()
	{
		global $post;
		$fortnox_order_number = get_post_meta($post->ID, '_uf_order_number', true);
		$fortnox_invoice_number = get_post_meta($post->ID, '_uf_invoice_number', true);
		$fortnox_invoice_sent = get_post_meta($post->ID, '_uf_invoice_sent', true);
		$fortnox_invoice_booked = get_post_meta($post->ID, '_uf_invoice_booked', true);
		$fortnox_invoice_paid = get_post_meta($post->ID, '_uf_invoice_paid', true);
		$fortnox_invoice_printed = get_post_meta($post->ID, '_uf_invoice_printed', true);
		$fortnox_invoice_pdf = get_post_meta($post->ID, '_uf_invoice_pdf', true);

		$html = "<table class='form-table uniwin-fortnox-form-table' style='margin-bottom: 10px;'>
            <tr>
            <th>Fortnox Order Number</th>
            <td>" . (($fortnox_order_number) ? $fortnox_order_number : '-') . "</td>
            </tr>
            <tr>
            <th>Invoice Number</th>
            <td>" . (($fortnox_invoice_number) ? $fortnox_invoice_number : '-') . "</td>
            </tr>
            <tr>
            <th>Invoice Sent</th>
            <td>" . (($fortnox_invoice_sent == '1') ? 'Yes' : '-') . "</td>
            </tr>
            <tr>
            <th>Invoice Booked</th>
            <td>" . (($fortnox_invoice_booked == '1') ? 'Yes' : '-') . "</td>
            </tr>
            <tr>
            <th>Invoice Paid</th>
            <td>" . (($fortnox_invoice_paid == '1') ? 'Yes' : '-') . "</td>
            </tr>
            <tr>
            <th>Invoice PDF</th>
            <td>" . (($fortnox_invoice_printed == '1') ? '<a download="' . $post->ID . '-label" href="data:application/pdf;base64,' . $fortnox_invoice_pdf . '">Download</a>' : '-') . "</td>
            </tr>
            </table>";

		echo $html;
		echo '
            <ul class="order_actions submitbox">
			<li class="wide" id="uf_actions" style="padding:0">
            <input type="hidden" name="uwf_meta_field_nonce" value="' . wp_create_nonce() . '">
            <select name="uwf_meta_action" style="width:219px">
            <option value="">Choose an action...</option>
                                    <option value="new_sync">New Sync</option>
                                    <option value="resync">Re Sync</option>
                                    <option value="create_invoice">Create Invoice</option>
                                    <option value="send_invoice">Send Invoice</option>
                                    <option value="book_invoice">Book Invoice</option>
                                    <option value="print_invoice">Print Invoice</option>
                            </select><button class="button wc-reload"><span>Apply</span></button></li>';
	}
	/**
	 * Order detail page meta box actions
	 **/
	public function uniwin_order_metabox_save($post_id)
	{
		if (!isset($_POST['uwf_meta_field_nonce'])) {
			return $post_id;
		}
		$nonce = $_REQUEST['uwf_meta_field_nonce'];
		if (!wp_verify_nonce($nonce)) {
			return $post_id;
		}
		$order = wc_get_order($post_id);
		switch ($_POST['uwf_meta_action']) {
			case 'new_sync':
				$new_sync = Uniwin_Fortnox_Helpers::syncorder($post_id);
				if (isset($new_sync['error'])) {
					UfAdminNotice::error('Uniwin Fortnox : ' . $new_sync['error'], true);
				} else {
					$order->update_meta_data('_uf_order_number', $new_sync['Order']['DocumentNumber']);
					$order->update_meta_data('_uf_order_url', $new_sync['Order']['@url']);
					$order->save();
					UfAdminNotice::success("Uniwin Fortnox : Order sync success", true);
				}
				break;
			case 'resync':
				$order_resync = Uniwin_Fortnox_Helpers::order_resync($post_id);
				if (isset($order_resync['error'])) {
					UfAdminNotice::error('Uniwin Fortnox : ' . $order_resync['error'], true);
				} else {
					UfAdminNotice::success("Uniwin Fortnox : Order re-sync success", true);
				}
				break;
			case 'create_invoice':
				$create_invoice = Uniwin_Fortnox_Helpers::create_invoice($post_id);
				if (isset($create_invoice['error'])) {
					UfAdminNotice::error('Uniwin Fortnox : ' . $create_invoice['error'], true);
				} else {
					$order->update_meta_data('_uf_invoice_number', $create_invoice['Order']['InvoiceReference']);
					$order->save();
					UfAdminNotice::success("Uniwin Fortnox : Create Invoice success", true);
				}
				break;
			case 'send_invoice':
				$send_invoice = Uniwin_Fortnox_Helpers::send_invoice($post_id);
				if (isset($send_invoice['error'])) {
					UfAdminNotice::error('Uniwin Fortnox : ' . $send_invoice['error'], true);
				} else {
					$order->update_meta_data('_uf_invoice_sent', '1');
					$order->save();
					UfAdminNotice::success("Uniwin Fortnox : Send Invoice success", true);
				}
				break;
			case 'book_invoice':
				$book_invoice = Uniwin_Fortnox_Helpers::book_invoice($post_id);
				if (isset($book_invoice['error'])) {
					UfAdminNotice::error('Uniwin Fortnox : ' . $book_invoice['error'], true);
				} else {
					$order->update_meta_data('_uf_invoice_booked', '1');
					$order->save();
					UfAdminNotice::success("Uniwin Fortnox : Book Invoice success", true);
				}
				break;
			case 'print_invoice':
				$print_invoice = Uniwin_Fortnox_Helpers::print_invoice($post_id);
				if (!isset($print_invoice['error'])) {
					$order->update_meta_data('_uf_invoice_printed', '1');
					$order->update_meta_data('_uf_invoice_pdf', $print_invoice);
					$order->save();
					UfAdminNotice::success("Uniwin Fortnox : Print Invoice success", true);
				} else {
					UfAdminNotice::error('Uniwin Fortnox : ' . $print_invoice['error'], true);
				}
				break;
			default:
				return $post_id;
		}
	}

	/**
	 * Product autosync callback function
	 **/

	public function uf_on_product_save($product_id)
	{
		if (isset($this->uf_productsettings->autosync) && $this->uf_productsettings->autosync == '1') {
			// get product auto sync status
			$autosync_status = $this->uf_productsettings->autosync_status;
			$product_status = get_post_status($product_id);
			// check if in array of autosync status
			if (in_array($product_status, $autosync_status)) {
				// check if product is variable product
				$product = wc_get_product($product_id);
				if ($product->is_type('variable')) {
					$children = $product->get_children();
					foreach ($children as $child_id) {
						$uf_article_number = get_post_meta($child_id, '_uf_article_number', true);
						$sku = get_post_meta($child_id, '_sku', true);
						if (!empty($uf_article_number) && $uf_article_number == $sku) {
							$article = Uniwin_Fortnox_Helpers::updateArticle($child_id, $product_id);
						} else {
							$article = Uniwin_Fortnox_Helpers::createArticle($child_id, $product_id);
						}
						if (empty($article['error'])) {
							UfAdminNotice::success('Uniwin Fortnox : Article sync success', true);
							update_post_meta($child_id, '_uf_article_number',  $article['Article']['ArticleNumber']);
						} else {
							UfAdminNotice::error('Uniwin Fortnox : ' . $article['error'], true);
						}
					}
				} else {
					$uf_article_number = get_post_meta($product_id, '_uf_article_number', true);
					$sku = get_post_meta($child_id, '_sku', true);
					if (!empty($uf_article_number) && $uf_article_number == $sku) {
						$article = Uniwin_Fortnox_Helpers::updateArticle($product_id);
					} else {
						$article = Uniwin_Fortnox_Helpers::createArticle($product_id);
					}
					if (empty($article['error'])) {
						UfAdminNotice::success('Uniwin Fortnox : Article sync success', true);
						update_post_meta($product_id, '_uf_article_number',  $article['Article']['ArticleNumber']);
					} else {
						UfAdminNotice::error('Uniwin Fortnox : ' . $article['error'], true);
					}
				}
			} else {
				return true;
			}
		}
	}

	/**
	 * Adds custom fields to the product page
	 **/

	public function uniwin_fortnox_product_action()
	{
		global $product_object;
		if (isset($this->uf_productsettings->sync_fields) && in_array('purchase_price', $this->uf_productsettings->sync_fields)) {
			echo '<div class=" product_custom_field ">';
			woocommerce_wp_text_input(array(
				'id'          => '_uf_purchase_price',
				'label'       => __('Purchase Price:', 'woocommerce'),
				'placeholder' => '',
				'desc_tip'    => 'true'
			));
			echo '</div>';
		}

		if (isset($this->uf_productsettings->sync_fields) && in_array('EAN', $this->uf_productsettings->sync_fields)) {
			echo '<div class=" product_custom_field ">';
			woocommerce_wp_text_input(array(
				'id'          => '_uf_ean',
				'label'       => __('EAN:', 'woocommerce'),
				'placeholder' => '',
				'desc_tip'    => 'true'
			));
			echo '</div>';
		}
		if (isset($this->uf_productsettings->sync_fields) && in_array('unit', $this->uf_productsettings->sync_fields)) {
			$units = Uniwin_Fortnox_Helpers::get_fortnox_units();
			$options = array('' => __('Select unit', 'woocommerce'));
			if (!empty($units['Units'])) {
				foreach ($units['Units'] as $unit) {
					$options[$unit['Code']] = __($unit['Code'] . '-' . $unit['Description'], 'woocommerce');
				}
			}

			echo '<div class=" product_custom_field ">';
			woocommerce_wp_select(array(
				'id'          => '_uf_product_unit',
				'label'       => __('Unit:', 'woocommerce'),
				'placeholder' => '',
				'desc_tip'    => 'true',
				'options'     => $options
			));


			echo '</div>';
		}
		if (isset($this->uf_productsettings->sync_fields) && in_array('manufacturer', $this->uf_productsettings->sync_fields)) {
			echo '<div class=" product_custom_field ">';
			woocommerce_wp_text_input(array(
				'id'          => '_uf_manufacturer',
				'label'       => __('Manufacturer:', 'woocommerce'),
				'placeholder' => '',
				'desc_tip'    => 'true'
			));
			echo '</div>';
		}

		if (isset($this->uf_productsettings->sync_fields) && in_array('manufacturer_article', $this->uf_productsettings->sync_fields)) {
			echo '<div class=" product_custom_field ">';
			woocommerce_wp_text_input(array(
				'id'          => '_uf_manufacturer_article',
				'label'       => __('Manufacturer Article Number:', 'woocommerce'),
				'placeholder' => '',
				'desc_tip'    => 'true'
			));
			echo '</div>';
		}
	}

	/**
	 * Product custom field save function
	 **/

	function uf_save_product_customfields($product)
	{
		if (isset($_POST['_uf_purchase_price'])) {
			$product->update_meta_data('_uf_purchase_price', sanitize_text_field($_POST['_uf_purchase_price']));
		}
		if (isset($_POST['_uf_ean'])) {
			$product->update_meta_data('_uf_ean', sanitize_text_field($_POST['_uf_ean']));
		}
		if (isset($_POST['_uf_product_unit'])) {
			$product->update_meta_data('_uf_product_unit', sanitize_text_field($_POST['_uf_product_unit']));
		}
		if (isset($_POST['_uf_manufacturer'])) {
			$product->update_meta_data('_uf_manufacturer', sanitize_text_field($_POST['_uf_manufacturer']));
		}
		if (isset($_POST['_uf_manufacturer_article'])) {
			$product->update_meta_data('_uf_manufacturer_article', sanitize_text_field($_POST['_uf_manufacturer_article']));
		}
	}

	/**
	 * Creates Log
	 **/

	public function uf_log_action($entry, $mode = 'a', $file = 'uf')
	{
		$upload_dir = UNIWINFORT_PLUGIN_DIR;
		if (is_array($entry)) {
			$entry = json_encode($entry);
		}
		$file  = $upload_dir . '/' . $file . '.log';
		$file  = fopen($file, $mode);
		$bytes = fwrite($file, current_time('mysql') . "::\n");
		$bytes = fwrite($file, $entry);
		$bytes = fwrite($file, ":::\n");
		fclose($file);
		return $bytes;
	}

	/*
	 * Cron schedule function for invoice
	 */
	public function schedule_invoice_task()
	{
		if (isset($this->uf_invoicesettings->change_orderstatus) && $this->uf_invoicesettings->change_orderstatus == '1') {
			if (!wp_next_scheduled('uwfx_autoinvoice_hook')) {
				wp_schedule_event(time(), '1min', 'uwfx_autoinvoice_hook');
			}
		}
	}

	public function uwfx_cron_schedules($schedules)
	{

		if (!isset($schedules["1min"])) {
			$schedules["1min"] = array(
				'interval' => 60,
				'display' => __('Once every 1 minutes')
			);
		}
		return $schedules;
	}

	/* this code handles the auto invoice and payment process for orders that have been synced and have a specific status from postnord. */

	public function uwfx_schedule_invoice_function()
	{
		$args = array(
			'status' => array('processing', 'pending', 'on-hold'),
			'meta_query' => array(
				array(
					'key' => '_uf_invoice_paid',
					'compare' => 'NOT EXISTS',
				),
				array(
					'key' => '_uf_invoice_number',
					'compare' => 'EXISTS',
				)
			)
		);
		$orders = wc_get_orders($args);
		if (!empty($orders)) {
			foreach ($orders as $order) {
				$fortnox_invoice_number = get_post_meta($order->get_id(), '_uf_order_number', true);
				$invoice = Uniwin_Fortnox_Helpers::get_invoice($fortnox_invoice_number);
				if (empty($invoice['error']) && $invoice['Invoice']['Balance'] == 0) {
					$order->update_meta_data('_uf_invoice_paid', '1');
					$order->update_status('completed');
					$note = 'Invoice' . $fortnox_invoice_number . ' paid. Order status updated to completed by Uniwin fortnox';
					$order->add_order_note($note);
				}
			}
		}

		$orders = wc_get_orders(array(
			'status' => array('processing', 'pending', 'on-hold'),
			'payment_method' => 'cod',
			'limit'        => -1, // Query all orders
			'orderby'      => 'date',
			'order'        => 'DESC',
			'meta_query' => array(
				array(
					'key' => '_uf_invoice_booked',
					'compare' => 'NOT EXISTS',
				),
				array(
					'key' => '_uf_invoice_number',
					'compare' => 'EXISTS',
				),
				array(
					'key' => '_payment_status',
					'value' => 'paid',
					'compare' => '!='
				)
			)
		));
		if (!empty($orders)) {
			foreach ($orders as $order) {
				$fortnox_invoice_number = get_post_meta($order->get_id(), '_uf_order_number', true);
				$invoice = Uniwin_Fortnox_Helpers::get_invoice($fortnox_invoice_number);
				if (empty($invoice['error'])) {
					if (!empty($invoice['Invoice']) && $invoice['Invoice']['Booked'] == true) {
						$order->update_meta_data('_uf_invoice_booked', '1');
						$order->payment_complete();
						$note = 'Invoice' . $fortnox_invoice_number . ' booked. Order status updated to paid by Uniwin Fortnox';
						$order->add_order_note($note);
					}
				}
			}
		}
		return true;
	}


	/**
	 * Enqueue the backend related scripts and styles for this plugin.
	 * All of the added scripts andstyles will be available on every page within the backend.
	 *
	 * @access	public
	 * @since	1.0.0
	 *
	 * @return	void
	 */
	public function enqueue_backend_scripts_and_styles()
	{
		wp_enqueue_style('uniwinfort-backend-styles', UNIWINFORT_PLUGIN_URL . 'core/includes/assets/css/backend-styles.css', array(), UNIWINFORT_VERSION, 'all');
		wp_enqueue_script('uniwinfort-backend-scripts', UNIWINFORT_PLUGIN_URL . 'core/includes/assets/js/backend-scripts.js', array(), UNIWINFORT_VERSION, false);
		wp_localize_script('uniwinfort-backend-scripts', 'uniwinfort', array(
			'plugin_name'   	=> __(UNIWINFORT_NAME, 'uniwin-fortnox'),
		));
	}

	/**
	 * ####################
	 * ### WP Webhooks 
	 * ####################
	 */

	/*
	 * Register dynamically all integrations
	 * The integrations are available within core/includes/integrations.
	 * A new folder is considered a new integration.
	 *
	 * @access	public
	 * @since	1.0.0
	 *
	 * @return	void
	 */
	public function add_wp_webhooks_integrations()
	{

		// Abort if WP Webhooks is not active
		if (!function_exists('WPWHPRO')) {
			return;
		}

		$custom_integrations = array();
		$folder = UNIWINFORT_PLUGIN_DIR . 'core' . DIRECTORY_SEPARATOR . 'includes' . DIRECTORY_SEPARATOR . 'integrations';

		try {
			$custom_integrations = WPWHPRO()->helpers->get_folders($folder);
		} catch (Exception $e) {
			WPWHPRO()->helpers->log_issue($e->getTraceAsString());
		}

		if (!empty($custom_integrations)) {
			foreach ($custom_integrations as $integration) {
				$file_path = $folder . DIRECTORY_SEPARATOR . $integration . DIRECTORY_SEPARATOR . $integration . '.php';
				WPWHPRO()->integrations->register_integration(array(
					'slug' => $integration,
					'path' => $file_path,
				));
			}
		}
	}
}
